#!/usr/bin/perl

## Dan Barrett
## 4/17/2009
## Program: texture_renewer.pl
## Purpose: Parses a .map and .shader file, reveals what texture files are in use, and what textures/shaders are in use.


## NOTES, USAGE:
## Put this script in the maps/ dir. Specify the map name (without the .map extension). The script does the rest.
## usage: texture_renewer.pl [map]

$map = "ut4_superman_b5";  # manually specify the map

$map = shift @ARGV if ($ARGV[0]);  # or specify one via CLI
$map =~ s/\.map$//;


open (MAP, "$map.map") || die ("$map.map not found");

while ($map_line = <MAP>) {
  chomp;
  $s_line =~ tr/A-Z/a-z/;
  ($texture) = $map_line =~ / ([A-Za-z0-9-_]+\/[A-Za-z0-9-_\/]+)/;
  $table{$texture}++;
}

close MAP;

open (SHADER, "../scripts/$map.shader") || die ("scripts/$map.shader not found");

while ($s_line = <SHADER>) {
  chomp;
  $s_line =~ tr/A-Z/a-z/;
  ($shader) = $s_line =~ /textures\/(\S+\/\S+)/;
  ($s_name,$s_ext) = split(/\./,$shader) if $shader =~ /\./;
  $table{$shader} = -1 if ! $s_ext;
  $table{$s_name} = $s_ext if $s_ext;
  $s_ext = "";
}

close MAP;

$dir = `dir /B ..\\textures\\$map`;

@files = split (/\s+/,$dir);

foreach $f_line (@files) {
  chomp;
  $f_line =~ tr/A-Z/a-z/;
  ($f_name,$f_ext) = split (/\./,$f_line);
  $f_tex = "$map/$f_name";
  $table{$f_tex} = $f_ext if $table{$f_tex} != 0;
}


print "\n\nActive texture dependencies:\n";
`del /S /Q ..\\textures\\$map\\renew`;
`mkdir ..\\textures\\$map\\renew`;
foreach $t_line (sort keys %table) {
  $t_ext = $table{$t_line};
  print "$t_line.$t_ext\n" if $t_ext !~ /\d+/;
  $t_line =~ s/\//\\/g;
  `xcopy /Q /Y /c ..\\textures\\$t_line.$t_ext ..\\textures\\$map\\renew` if $t_ext !~ /\d+/;
}

print "\n\nInactive textures:\n";
foreach $i_line (sort @files) {
  chomp;
  $i_line =~ tr/A-Z/a-z/;
  ($i_name,$i_ext) = split (/\./,$i_line);
  $i_tex = "$map/$i_name";
  print "$i_tex.$i_ext\n" if $table{$i_tex} ne $i_ext;
}



print "\n\nExternal shaders:\n";
foreach $t_line (sort keys %table) {
  $t_ext = $table{$t_line};
  next if $t_line !~ /\S+/;
  print "$t_line\n" if $t_ext =~ /\d+/ && $t_ext > 0;
}



print "\n\nLocal shaders:\n";
foreach $t_line (sort keys %table) {
  $t_ext = $table{$t_line};
  next if $t_line !~ /\S+/;
  print "$t_line\n" if $t_ext =~ /\d+/ && $t_ext < 0;
}

